Sexp

Struct Sexp 

pub struct Sexp<T: RType> { /* private fields */ }
Available on crate feature public-all only.
ReadOnly SEXP, should not be changed.

This is the preferred way for accept ffi values Since R marked the input values as readonly.

Implementations

Source

impl<T: RType> Sexp<T>

pub const unsafe fn missing(self) -> bool

👎Deprecated: Sexp<T> cannot missing (while the item is registered), using OptionSexp<T> instead.

Sexp means the item is not missing, using OptionSexp<T> if the sexp is optional (The best practice is, never use this!)

Source

impl Sexp<Rchar>

pub unsafe fn error(self) -> !

REALLY UNSAFE FUNCTION

see error for more informations.

pub fn print(self)

print the content to R… with R’s print function.

It is more preferred than std::println! since output with std::println! might be ignored with windows Rgui.exe.

pub fn eprint(self)

print the content to R… with R’s eprint function.