Struct rmin::Owned

source ·
pub struct Owned<T: RType> { /* private fields */ }
Owned SEXP, allocated by Rust code.

You could modify object as a slice object, and return it as a R vector. Once it is created, you should take care about calling R function. The ideal way is call R functions before it is allocated (e.g., convert all SEXP<T> into &[T]) then allocate only 1 Owned object.

Implementations

source

impl<T: RType> Owned<T>

source

pub fn protect(self) -> Protected<T>

This function provides a way to protect any vector from R’s GC You should take care about that, any allocation might recycle the previous allocated vector In the current fasion, we only alloc 1 return vector, thus protect is not necessary In case you want to communicate with R functions, you must convert all your Owned values to the Protected version.

Examples found in repository?
examples/compare-rmin.rs (line 6)
4
5
6
7
8
9
10
pub extern "C-unwind" fn add_protect(a:SEXP<f64>,b:SEXP<f64>) -> Owned<f64> {
    handle_panic(||{
        let mut c=Owned::new(1).protect();
        c[0]=a[0]+b[0];
        c.into()
    })
}