pub struct Sexp<T: RType> { /* private fields */ }Available on crate feature
public-all only.Expand description
ReadOnly SEXP, should not be changed.
This is the preferred way for accept ffi values
Since R marked the input values as readonly.
Implementations
Sourceimpl<T: RType> Sexp<T>
impl<T: RType> Sexp<T>
Sourcepub const unsafe fn missing(self) -> bool
👎Deprecated: Sexp<T> cannot missing (while the item is registered), using OptionSexp<T> instead.
pub const unsafe fn missing(self) -> bool
Sexp means the item is not missing, using OptionSexp<T> if the sexp is optional
(The best practice is, never use this!)